# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/25_training.core.ipynb.

# %% ../nbs/25_training.core.ipynb 3
from __future__ import annotations
import numpy as np
import pandas as pd

# %% auto 0
__all__ = ['is_plateau']

# %% ../nbs/25_training.core.ipynb 5
def is_plateau(array:np.ndarray, # 1D array to test
               w_size:int=3, # Moving average window size
               delta:float=0.01, # Threshold indicating plateau  
               verbose:bool=True # Display mean's evolution
              ):
    '''Detect if a plateau is reached'''  
    if len(array) < w_size:
        return False
    prev_mean = np.mean(array[-(w_size+1):-1])
    new_mean = np.mean(array[-w_size:])
    if verbose:
        print(f'Previous mean: {prev_mean:.3f}')
        print(f'New mean: {new_mean:.3f}')
        print(f'Is means difference: {prev_mean - new_mean:.3f} < {delta:.3f}')
    return True if (prev_mean - new_mean) < delta else False



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/04_data.selection.ipynb.

# %% auto 0
__all__ = ['select_y', 'select_tax_order', 'select_X', 'get_y_by_order']

# %% ../nbs/04_data.selection.ipynb 3
#nbdev_comment from __future__ import annotations
from pathlib import Path
import pickle
import numpy as np
from fastcore.test import *

# %% ../nbs/04_data.selection.ipynb 4
def select_y(data:tuple, # (X, y, X_id, depth_order)
             low:float=0.12, # Lowest limit
             high:float=999 # Highest limit
            ):
    """Select data based on the limit values of the target"""
    X, y, X_id, depth_order = data
    idx = np.logical_and((y >= low), (y <= high))
    return X[idx, :], y[idx], X_id[idx], depth_order[idx, :]

# %% ../nbs/04_data.selection.ipynb 5
def select_tax_order(data:tuple, # (X, y, X_id, depth_order)
                     tax_order:int=None # Value between 0 and 12
                    ):
    """Select data based on Soil Taxonomy order"""
    X, y, X_id, depth_order = data
    if tax_order:
        idx = depth_order[:, 1] == tax_order
        return X[idx, :], y[idx], X_id[idx], depth_order[idx, :]
    else:
        return data

# %% ../nbs/04_data.selection.ipynb 6
def select_X(data:tuple, # (X, y, X_id, depth_order)
             low:int=0 # Lowest absorbance value
            ):
    """Select data based on the limit values (only low) of the features"""
    X, y, X_id, depth_order = data
    idx = np.min(X, axis=1) > low
    return X[idx, :], y[idx], X_id[idx], depth_order[idx, :]

# %% ../nbs/04_data.selection.ipynb 7
def get_y_by_order(y, tax_order, tax_lookup):
    tax_values, count_by_order = np.unique(tax_order, return_counts=True)
    idx_order = count_by_order.argsort()

    y_by_order = []
    for tax in tax_values[idx_order]:
        mask_order = tax_order == tax
        y_by_order.append(y[mask_order])

    return (y_by_order, count_by_order[idx_order], idx_order)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/03_vis.core.ipynb.

# %% auto 0
__all__ = ['PRIMARY_COLOR', 'ACCENT_COLOR', 'DEFAULT_STYLE', 'centimeter', 'set_style', 'plot_spectra', 'summary_plot',
           'plot_validation_curve', 'plot_learning_curve', 'plot_capacity']

# %% ../nbs/03_vis.core.ipynb 3
#nbdev_comment from __future__ import annotations
from ..data.loading import load_kssl
from ..data.selection import get_y_by_order
import numpy as np

import matplotlib.pyplot as plt
from matplotlib.ticker import EngFormatter
from matplotlib import ticker
from fastcore.test import *

# %% ../nbs/03_vis.core.ipynb 5
PRIMARY_COLOR = '#333'
ACCENT_COLOR = 'firebrick'
DEFAULT_STYLE = {
    'axes.linewidth': 0.5,
    'axes.facecolor': 'white',
    'axes.ymargin': 0.11,
    'font.size': 8,
    
    'axes.spines.bottom': True,
    'axes.spines.left': False,
    'axes.spines.right': False,
    'axes.spines.top': False,
    'axes.grid': True,
    
    'grid.color': 'black',
    'grid.linewidth': 0.2,
    'grid.linestyle': '-',

    'xtick.bottom': True,
    'xtick.top': False,
    'xtick.direction': 'out',
    'xtick.major.size': 5,
    'xtick.major.width': 1,
    'xtick.minor.size': 3,
    'xtick.minor.width': 0.5,
    'xtick.minor.visible': True,
        
    'ytick.left': True,
    'ytick.right': False, 
    'ytick.direction': 'in',
    'ytick.major.size': 5,
    'ytick.major.width': 1,
    'ytick.minor.size': 3,
    'ytick.minor.width': 0.5,
    'ytick.minor.visible': True
}

centimeter = 1/2.54  # centimeters in inches

# %% ../nbs/03_vis.core.ipynb 6
def set_style(style:dict # Dictionary of plt.rcParams
             ):
    for k, v in style.items():
        plt.rcParams[k] = v 

# %% ../nbs/03_vis.core.ipynb 8
def plot_spectra(X:np.ndarray, # Spectra (n_samples, n_wavenumbers)
                 X_names:np.ndarray, # Wavenumbers (n_wavenumbers)
                 figsize=(18, 5), # Wavenumbers
                 sample=20): # Size of random subset
    """Plot Mid-infrared spectra"""
    fig, ax = plt.subplots(figsize=figsize)
    idx = np.random.randint(X.shape[0], size=sample)
    ax.set_xlim(np.max(X_names), np.min(X_names))
    ax.set(xlabel='Wavenumber', ylabel='Absorbance')
    ax.set_axisbelow(True)
    ax.grid(True, which='both')
    _ = ax.plot(X_names, X[idx, :].T)

# %% ../nbs/03_vis.core.ipynb 9
def summary_plot(y:np.ndarray, # Target variable (n_samples)
                 depth_order:np.ndarray, # Soil and Depth (n_samples, 2)
                 tax_lookup:dict, # {'alfisols': 0,'mollisols': 1, ...}
                ):
    p = plt.rcParams
    p["font.size"] = 8

    p["axes.linewidth"] = 1
    p["axes.facecolor"] = "white"
    p["axes.ymargin"] = 0.1
    p["axes.spines.bottom"] = True
    p["axes.spines.left"] = False
    p["axes.spines.right"] = False
    p["axes.spines.top"] = False

    p["axes.grid"] = True
    p["grid.color"] = "black"
    p["grid.linewidth"] = 0.2
    p['grid.linestyle'] = '--'

    p["ytick.left"] = True
    p["ytick.right"] = True
    p["ytick.major.size"] = 0
    p["ytick.major.width"] = 1
    p["ytick.minor.size"] = 0
    p["ytick.minor.width"] = 0.5
    p["ytick.minor.visible"] = False

    fig, (ax1, ax2) = plt.subplots(nrows=1,ncols=2, gridspec_kw={'width_ratios': [2, 2]},
                                   sharey=True, figsize=(15*centimeter, 8*centimeter), dpi=600)

    y_by_order, count_by_order, idx_order = get_y_by_order(y, depth_order[:, 1], tax_lookup)
    y_labels = np.array([k.capitalize() for k, v in tax_lookup.items()])[idx_order]

    rects = ax1.barh(y_labels, count_by_order,
                     align='center',
                     height=0.65,
                     color=PRIMARY_COLOR)


    for i, v in enumerate(count_by_order):
        offset = 100 if i < len(count_by_order)-1 else -4000
        color = PRIMARY_COLOR if i < len(count_by_order)-1 else "white"
        ax1.text(v + offset, i - 0.01 , str(v),
                verticalalignment='center',
                horizontalalignment='right',
                color=color, fontweight='normal', size=6)

    for ax in [ax1, ax2]:
        ax.xaxis.set_major_locator(ticker.MaxNLocator(4))
        ax.xaxis.set_minor_locator(ticker.MaxNLocator(20))


    ax1.tick_params(axis='y', which='major', pad=30)
    ax1.set_xlabel('← Number of samples', loc='left')
    ax1.set_ylabel('Taxonomic order')
    formatter1 = EngFormatter(places=0, sep="\N{THIN SPACE}")  # U+2009
    ax1.xaxis.set_major_formatter(formatter1)
    ax1.set_yticklabels(y_labels, fontdict={'horizontalalignment': 'center'})
    ax1.yaxis.tick_right()
    ax1.invert_xaxis()
    ax1.set_title('(a)', loc='left')

    boxplot = ax2.boxplot(y_by_order, sym='.', positions=range(13), vert=False,
                        patch_artist=True)
    ax2.set_xlabel('Exchangeable Potassium ($cmol(+)kg^{-1}$) →', loc='right')

    for median in boxplot['medians']:
        median.set(color='white', linewidth=1)

    for box in boxplot['boxes']:
            box.set(facecolor=PRIMARY_COLOR)

    for flier in boxplot['fliers']:
        flier.set(markersize='1.5', markeredgecolor="tab:red", alpha=0.3, zorder=-1)

    ax2.set_xscale('log')
    ax2.set_title('(b)', loc='left')
    ax2.yaxis.set_ticks_position('none')

    plt.tight_layout()
    #plt.savefig(os.path.join(IMG_PATH, 'data-summary.png'), dpi=600, transparent=True, format='png')

# %% ../nbs/03_vis.core.ipynb 12
def plot_validation_curve(x, losses, ax=None, plot_kwargs={}, fill_between_kwargs={}):
    Y = np.mean(np.array(losses), axis=0)
    SD = np.std(np.array(losses), axis=0) 
    ax.fill_between(x, Y + SD, Y - SD, **fill_between_kwargs)
    ax.plot(x, Y, **plot_kwargs) 
    return(ax)

# %% ../nbs/03_vis.core.ipynb 14
def plot_learning_curve(x, losses_train, losses_valid, ax=None,  train_kwargs={}, valid_kwargs={}):
    if ax is None:
        ax = plt.gca()
    ax.plot(x, losses_train, label='Training', **train_kwargs) 
    ax.plot(x, losses_valid, label='Validation', **valid_kwargs) 
    ax.set_yscale('log')
    ax.set_xscale('log')
    return(ax)

# %% ../nbs/03_vis.core.ipynb 15
def plot_capacity(x, capacity, ax=None, **kwargs):
    if ax is None:
            ax = plt.gca()
    ax.bar(x, capacity, width=0.15*np.array(x), color=PRIMARY_COLOR, zorder=99, **kwargs)
    ax.set_yscale('log')
    ax.set_xscale('log')
    # ax.spines.bottom.set_visible(True) 
    return(ax)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/01_data.loading.ipynb.

# %% auto 0
__all__ = ['load_kssl']

# %% ../nbs/01_data.loading.ipynb 3
#nbdev_comment from __future__ import annotations
from pathlib import Path
import pickle
import numpy as np
from fastcore.test import *
from typing import List

# %% ../nbs/01_data.loading.ipynb 4
def load_kssl(src_dir:str, # folder path containing data
              fnames:List[str]=['spectra-features.npy', 'spectra-wavenumbers.npy', # filenames to open (in order)
                                'depth-order.npy', 'target.npy',
                                'tax-order-lu.pkl', 'spectra-id.npy'],
             loaders_lut:dict={'.npy': np.load, '.pkl': pickle.load} # loaders lookup table
             ):
    """
    Function loading USDA KSSL dataset focusing here on Exchangeable Potassium (analyte_id=725).

    Returns:
        A tuple (X, X_names, depth_order, y, tax) with:
            X: spectra (numpy.ndarray)
            X_names: spectra wavenumbers (numpy.ndarray)
            depth_order: depth and order of samples (numpy.ndarray)
            y: exchangeable potassium content (numpy.ndarray)
            tax_lookup: look up table order_id -> order_name (Dictionary)
            X_id: unique id of spectra
    """
    fnames = [Path(src_dir)/fname for fname in fnames]
    loaders = [loaders_lut[fname.suffix] for fname in fnames]
    return [loader(open(fname, 'rb')) for loader, fname in zip(loaders, fnames)]



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/13_data.torch.ipynb.

# %% auto 0
__all__ = ['SpectralDataset', 'DataLoaders', 'SNV_transform', 'Noop']

# %% ../nbs/13_data.torch.ipynb 3
#nbdev_comment from __future__ import annotations
import numpy as np

from fastcore.test import *

from .loading import load_kssl
from .selection import (select_y, select_tax_order, select_X)
from .transform import (log_transform_y, SNV)

from sklearn.model_selection import train_test_split

from fastcore.transform import compose

import torch
from torch import nn
from torchvision import transforms
from torch.utils.data import DataLoader, Dataset

# %% ../nbs/13_data.torch.ipynb 5
class SpectralDataset(Dataset):
    def __init__(self, X, y, tax_order, transform=None):
        self.X = X
        self.y = y
        self.tax_order = tax_order
        self.transform = transform

    def __len__(self):
        return len(self.y)

    def __getitem__(self, idx):
        X = self.X[None, idx, :]
        y = self.y[None, idx]
        tax_order = self.tax_order[None, idx]
        if self.transform:
            X = self.transform(X)
        return X.astype(np.float32), y.astype(np.float32), tax_order.astype(np.intc)

# %% ../nbs/13_data.torch.ipynb 6
class DataLoaders():
    def __init__(self, data, transform=None, batch_size=32):
        """
        Convert numpy error to Pytorch data loaders (generators)
        Args:
            data: tuple as ((X_train, y_train), (X_test, y_test))
            transform: callable class (__class__)

        Returns:
            (training_generator, validation_generator)
        """
        self.data = data
        self.batch_size = batch_size
        self.transform = transform if transform else Noop()

    def loaders(self):
        return (DataLoader(SpectralDataset(X, y, tax_order, transform=self.transform), batch_size=self.batch_size)
                for X, y, tax_order in self.data)

# %% ../nbs/13_data.torch.ipynb 8
class SNV_transform():
    def __init__(self):
        None
    def __call__(self, spectrum):
        return SNV().fit_transform(spectrum)

# %% ../nbs/13_data.torch.ipynb 9
class Noop():
    def __init__(self):
        None
    def __call__(self, X):
        return X



# Autogenerated by nbdev

d = { 'settings': { 'allowed_cell_metadata_keys': '',
                'allowed_metadata_keys': '',
                'audience': 'Developers',
                'author': 'Franck Albinet',
                'author_email': 'franckalbinet@gmail.com',
                'black_formatting': 'False',
                'branch': 'main',
                'clean_ids': 'True',
                'copyright': 'Franck Albinet',
                'custom_quarto_yml': 'True',
                'custom_sidebar': 'True',
                'description': 'Prediction of Exchangeable Potassium in Soil through Mid-Infrared and Deep Learning paper code',
                'doc_baseurl': '/mirzai/',
                'doc_host': 'https://franckalbinet.github.io',
                'doc_path': '_docs',
                'git_url': 'https://github.com/franckalbinet/mirzai/',
                'host': 'github',
                'jupyter_hooks': 'True',
                'keywords': 'high-throughput soil characterization remediation',
                'language': 'English',
                'lib_name': 'mirzai',
                'lib_path': 'mirzai',
                'license': 'apache2',
                'min_python': '3.7',
                'nbs_path': 'nbs',
                'readme_nb': 'index.ipynb',
                'recursive': 'False',
                'repo': 'mirzai',
                'requirements': 'numpy pandas fastcore scipy matplotlib>=3.5.1 scikit-learn torch torchvision tqdm captum',
                'status': '2',
                'title': 'mirzai',
                'tst_flags': 'notest',
                'user': 'franckalbinet',
                'version': '0.2.0'},
  'syms': { 'mirzai.data.loading': {'mirzai.data.loading.load_kssl': 'https://franckalbinet.github.io/mirzai/data.loading.html#load_kssl'},
            'mirzai.data.selection': { 'mirzai.data.selection.get_y_by_order': 'https://franckalbinet.github.io/mirzai/data.selection.html#get_y_by_order',
                                       'mirzai.data.selection.select_X': 'https://franckalbinet.github.io/mirzai/data.selection.html#select_x',
                                       'mirzai.data.selection.select_tax_order': 'https://franckalbinet.github.io/mirzai/data.selection.html#select_tax_order',
                                       'mirzai.data.selection.select_y': 'https://franckalbinet.github.io/mirzai/data.selection.html#select_y'},
            'mirzai.data.torch': { 'mirzai.data.torch.DataLoaders': 'https://franckalbinet.github.io/mirzai/data.torch.html#dataloaders',
                                   'mirzai.data.torch.DataLoaders.loaders': 'https://franckalbinet.github.io/mirzai/data.torch.html#dataloaders.loaders',
                                   'mirzai.data.torch.Noop': 'https://franckalbinet.github.io/mirzai/data.torch.html#noop',
                                   'mirzai.data.torch.SNV_transform': 'https://franckalbinet.github.io/mirzai/data.torch.html#snv_transform',
                                   'mirzai.data.torch.SpectralDataset': 'https://franckalbinet.github.io/mirzai/data.torch.html#spectraldataset'},
            'mirzai.data.transform': { 'mirzai.data.transform.CO2_REGION': 'https://franckalbinet.github.io/mirzai/data.transform.html#co2_region',
                                       'mirzai.data.transform.DropSpectralRegions': 'https://franckalbinet.github.io/mirzai/data.transform.html#dropspectralregions',
                                       'mirzai.data.transform.DropSpectralRegions.fit': 'https://franckalbinet.github.io/mirzai/data.transform.html#dropspectralregions.fit',
                                       'mirzai.data.transform.DropSpectralRegions.transform': 'https://franckalbinet.github.io/mirzai/data.transform.html#dropspectralregions.transform',
                                       'mirzai.data.transform.SNV': 'https://franckalbinet.github.io/mirzai/data.transform.html#snv',
                                       'mirzai.data.transform.SNV.fit': 'https://franckalbinet.github.io/mirzai/data.transform.html#snv.fit',
                                       'mirzai.data.transform.SNV.transform': 'https://franckalbinet.github.io/mirzai/data.transform.html#snv.transform',
                                       'mirzai.data.transform.TakeDerivative': 'https://franckalbinet.github.io/mirzai/data.transform.html#takederivative',
                                       'mirzai.data.transform.TakeDerivative.fit': 'https://franckalbinet.github.io/mirzai/data.transform.html#takederivative.fit',
                                       'mirzai.data.transform.TakeDerivative.transform': 'https://franckalbinet.github.io/mirzai/data.transform.html#takederivative.transform',
                                       'mirzai.data.transform.log_transform_y': 'https://franckalbinet.github.io/mirzai/data.transform.html#log_transform_y'},
            'mirzai.training.cnn': { 'mirzai.training.cnn.Evaluator': 'https://franckalbinet.github.io/mirzai/training.cnn.html#evaluator',
                                     'mirzai.training.cnn.Evaluator.eval_on_test': 'https://franckalbinet.github.io/mirzai/training.cnn.html#evaluator.eval_on_test',
                                     'mirzai.training.cnn.Evaluator.eval_on_train': 'https://franckalbinet.github.io/mirzai/training.cnn.html#evaluator.eval_on_train',
                                     'mirzai.training.cnn.Evaluator.reduce': 'https://franckalbinet.github.io/mirzai/training.cnn.html#evaluator.reduce',
                                     'mirzai.training.cnn.Evaluator.train_multiple': 'https://franckalbinet.github.io/mirzai/training.cnn.html#evaluator.train_multiple',
                                     'mirzai.training.cnn.Learner': 'https://franckalbinet.github.io/mirzai/training.cnn.html#learner',
                                     'mirzai.training.cnn.Learner.eval': 'https://franckalbinet.github.io/mirzai/training.cnn.html#learner.eval',
                                     'mirzai.training.cnn.Learner.fit': 'https://franckalbinet.github.io/mirzai/training.cnn.html#learner.fit',
                                     'mirzai.training.cnn.Learner.predict': 'https://franckalbinet.github.io/mirzai/training.cnn.html#learner.predict',
                                     'mirzai.training.cnn.Model': 'https://franckalbinet.github.io/mirzai/training.cnn.html#model',
                                     'mirzai.training.cnn.Model.forward': 'https://franckalbinet.github.io/mirzai/training.cnn.html#model.forward',
                                     'mirzai.training.cnn.Model.make_convpool_block': 'https://franckalbinet.github.io/mirzai/training.cnn.html#model.make_convpool_block',
                                     'mirzai.training.cnn.weights_init': 'https://franckalbinet.github.io/mirzai/training.cnn.html#weights_init'},
            'mirzai.training.core': {'mirzai.training.core.is_plateau': 'https://franckalbinet.github.io/mirzai/training.core.html#is_plateau'},
            'mirzai.training.metrics': { 'mirzai.training.metrics.eval_reg': 'https://franckalbinet.github.io/mirzai/training.metrics.html#eval_reg',
                                         'mirzai.training.metrics.lccc': 'https://franckalbinet.github.io/mirzai/training.metrics.html#lccc',
                                         'mirzai.training.metrics.mape': 'https://franckalbinet.github.io/mirzai/training.metrics.html#mape',
                                         'mirzai.training.metrics.rpd': 'https://franckalbinet.github.io/mirzai/training.metrics.html#rpd',
                                         'mirzai.training.metrics.rpiq': 'https://franckalbinet.github.io/mirzai/training.metrics.html#rpiq',
                                         'mirzai.training.metrics.stb': 'https://franckalbinet.github.io/mirzai/training.metrics.html#stb'},
            'mirzai.training.plsr': { 'mirzai.training.plsr.Evaluator': 'https://franckalbinet.github.io/mirzai/training.plsr.html#evaluator',
                                      'mirzai.training.plsr.Evaluator.eval_on_test': 'https://franckalbinet.github.io/mirzai/training.plsr.html#evaluator.eval_on_test',
                                      'mirzai.training.plsr.Evaluator.eval_on_train': 'https://franckalbinet.github.io/mirzai/training.plsr.html#evaluator.eval_on_train',
                                      'mirzai.training.plsr.Evaluator.reduce': 'https://franckalbinet.github.io/mirzai/training.plsr.html#evaluator.reduce',
                                      'mirzai.training.plsr.Evaluator.train_multiple': 'https://franckalbinet.github.io/mirzai/training.plsr.html#evaluator.train_multiple',
                                      'mirzai.training.plsr.Learner': 'https://franckalbinet.github.io/mirzai/training.plsr.html#learner',
                                      'mirzai.training.plsr.Learner.evaluate': 'https://franckalbinet.github.io/mirzai/training.plsr.html#learner.evaluate',
                                      'mirzai.training.plsr.Learner.find_hp': 'https://franckalbinet.github.io/mirzai/training.plsr.html#learner.find_hp',
                                      'mirzai.training.plsr.Learner.fit': 'https://franckalbinet.github.io/mirzai/training.plsr.html#learner.fit',
                                      'mirzai.training.plsr.PLS_model': 'https://franckalbinet.github.io/mirzai/training.plsr.html#pls_model',
                                      'mirzai.training.plsr.PLS_model.eval': 'https://franckalbinet.github.io/mirzai/training.plsr.html#pls_model.eval',
                                      'mirzai.training.plsr.PLS_model.fit': 'https://franckalbinet.github.io/mirzai/training.plsr.html#pls_model.fit',
                                      'mirzai.training.plsr.PLS_model.predict': 'https://franckalbinet.github.io/mirzai/training.plsr.html#pls_model.predict',
                                      'mirzai.training.plsr.compute_valid_curve': 'https://franckalbinet.github.io/mirzai/training.plsr.html#compute_valid_curve'},
            'mirzai.vis.core': { 'mirzai.vis.core.ACCENT_COLOR': 'https://franckalbinet.github.io/mirzai/vis.core.html#accent_color',
                                 'mirzai.vis.core.DEFAULT_STYLE': 'https://franckalbinet.github.io/mirzai/vis.core.html#default_style',
                                 'mirzai.vis.core.PRIMARY_COLOR': 'https://franckalbinet.github.io/mirzai/vis.core.html#primary_color',
                                 'mirzai.vis.core.centimeter': 'https://franckalbinet.github.io/mirzai/vis.core.html#centimeter',
                                 'mirzai.vis.core.plot_capacity': 'https://franckalbinet.github.io/mirzai/vis.core.html#plot_capacity',
                                 'mirzai.vis.core.plot_learning_curve': 'https://franckalbinet.github.io/mirzai/vis.core.html#plot_learning_curve',
                                 'mirzai.vis.core.plot_spectra': 'https://franckalbinet.github.io/mirzai/vis.core.html#plot_spectra',
                                 'mirzai.vis.core.plot_validation_curve': 'https://franckalbinet.github.io/mirzai/vis.core.html#plot_validation_curve',
                                 'mirzai.vis.core.set_style': 'https://franckalbinet.github.io/mirzai/vis.core.html#set_style',
                                 'mirzai.vis.core.summary_plot': 'https://franckalbinet.github.io/mirzai/vis.core.html#summary_plot'}}}